package com.gydx.controller;

import com.gydx.DTO.UserDTO;
import com.gydx.DTO.UserLoginDTO;
import com.gydx.VO.UserVO;
import com.gydx.constant.JwtClaimsConstant;
import com.gydx.entity.User;
import com.gydx.properties.JwtProperties;
import com.gydx.result.Result;
import com.gydx.service.UserService;
import com.gydx.util.JwtUtil;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.websocket.server.PathParam;
import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@Slf4j
@RequestMapping("/users")
@Tag(name = "body参数")
public class UserController {

    @Autowired
    private UserService userService;

    @Autowired
    private JwtProperties jwtProperties;


    /**
     * 用户登陆
     * @param userDTO
     * @return
     */
    @Operation(summary = "登陆")
    @PostMapping("/login")
    public Result login(@RequestBody UserLoginDTO userDTO){
        log.info("员工登陆:{}",userDTO);
        User user1 = userService.login(userDTO);

        //登录成功后，生成jwt令牌
        Map<String, Object> claims = new HashMap<>();
        claims.put(JwtClaimsConstant.USER_ID, user1.getId());
        String token = JwtUtil.createJWT(
                jwtProperties.getAdminSecretKey(),
                jwtProperties.getAdminTtl(),
                claims);

        UserVO userVO = UserVO.builder()
                .id(user1.getId())
                .userName(user1.getUsername())
                .name(user1.getName())

                .token(token)
                .build();

        return Result.success(userVO);
    }


    /**
     * 添加用户
     */
    @PostMapping("/add")
    public Result save(@RequestBody UserDTO userDTO){
        log.info("新增员工:{}",userDTO.toString());
        userService.save(userDTO);
        return Result.success();
    }


    /**
     * 删除用户
     */
    @DeleteMapping("/delete")
    public Result delete(@RequestParam List<Long> ids){
        log.info("用户的批量删除:{}",ids);
        userService.deleteByIds(ids);
        return Result.success();
    }

}
