package com.gydx.service.impl;


import com.gydx.DTO.UserDTO;
import com.gydx.DTO.UserLoginDTO;
import com.gydx.constant.MessageConstant;
import com.gydx.constant.PasswordConstant;
import com.gydx.constant.StatusConstant;
import com.gydx.entity.User;
import com.gydx.exception.AccountLockedException;
import com.gydx.exception.AccountNotFoundException;
import com.gydx.exception.PasswordErrorException;
import com.gydx.mapper.UserMapper;
import com.gydx.service.UserService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;

import java.util.List;

@Service
public class UserServiceImpl implements UserService {

    private static final Logger log = LoggerFactory.getLogger(UserServiceImpl.class);
    @Autowired
    private UserMapper userMapper;
    @Override
    public User login(UserLoginDTO userDTO){
        String username = userDTO.getUsername();
        String password = userDTO.getPassword();
        //1、根据用户名查询数据库中的数据
        User user = userMapper.getByUsername(username);

        //2、处理各种异常情况（用户名不存在、密码不对、账号被锁定）
        if (user == null) {
            //账号不存在
            throw new AccountNotFoundException(MessageConstant.ACCOUNT_NOT_FOUND);
        }

        //密码比对
        //对前端传过来的明文密码进行md5加密处理
        password = DigestUtils.md5DigestAsHex(password.getBytes());
        if (!password.equals(user.getPassword())) {
            //密码错误
            throw new PasswordErrorException(MessageConstant.PASSWORD_ERROR);
        }

        if (user.getStatus() == StatusConstant.DISABLE) {
            //账号被锁定
            throw new AccountLockedException(MessageConstant.ACCOUNT_LOCKED);
        }

        //3、返回实体对象
        return user;
    }

    @Override
    public void save(UserDTO userDTO) {
        User user = new User();
        String password = userDTO.getPassword();
        //对象的属性拷贝
        BeanUtils.copyProperties(userDTO, user);

        if (password == null || password.length() == 0) {
            user.setPassword(DigestUtils.md5DigestAsHex(PasswordConstant.DEFAULT_PASSWORD.getBytes()));
        }

        userMapper.insert(user);

    }

    @Override
    public void deleteByIds(List<Long> ids) {
        log.info("批量删除用户:{}",ids);
        userMapper.deleteByIds(ids);
    }
}
